@@ -15,6 +15,7 @@ import contextlib
 import copy
 import fixtures
 import pickle
+import sys
 import warnings
 
 import mock
@@ -36,6 +37,7 @@ from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import engines as oslo_engines
 from oslo_db.sqlalchemy import orm
 from oslo_db.tests.sqlalchemy import base as test_base
+from oslo_db import warning
 
 
 enginefacade.transaction_context_provider(oslo_context.RequestContext)
@@ -478,6 +480,32 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             [mock.call.dispose()]
         )
 
+    def test_started_flag(self):
+        facade = enginefacade.transaction_context()
+
+        self.assertFalse(facade.is_started)
+        facade.configure(connection=self.engine_uri)
+        facade.writer.get_engine()
+
+        self.assertTrue(facade.is_started)
+
+    def test_started_exception(self):
+        facade = enginefacade.transaction_context()
+
+        self.assertFalse(facade.is_started)
+        facade.configure(connection=self.engine_uri)
+        facade.writer.get_engine()
+
+        exc = self.assertRaises(
+            enginefacade.AlreadyStartedError,
+            facade.configure,
+            connection=self.engine_uri
+        )
+        self.assertEqual(
+            "this TransactionFactory is already started",
+            exc.args[0]
+        )
+
     def test_session_reader_decorator(self):
         context = oslo_context.RequestContext()
 
@@ -607,7 +635,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
 
     def test_async_on_writer_raises(self):
         exc = self.assertRaises(
-            TypeError, getattr, enginefacade.writer, "async"
+            TypeError, getattr, enginefacade.writer, "async_"
         )
         self.assertEqual(
             "Setting async on a WRITER makes no sense",
@@ -626,7 +654,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
     def test_reader_nested_in_async_reader_raises(self):
         context = oslo_context.RequestContext()
 
-        @enginefacade.reader.async
+        @enginefacade.reader.async_
         def go1(context):
             context.session.execute("test1")
             go2(context)
@@ -647,7 +675,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
     def test_reader_allow_async_nested_in_async_reader(self):
         context = oslo_context.RequestContext()
 
-        @enginefacade.reader.async
+        @enginefacade.reader.async_
         def go1(context):
             context.session.execute("test1")
             go2(context)
@@ -701,7 +729,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
     def test_writer_nested_in_async_reader_raises(self):
         context = oslo_context.RequestContext()
 
-        @enginefacade.reader.async
+        @enginefacade.reader.async_
         def go1(context):
             context.session.execute("test1")
             go2(context)
@@ -741,10 +769,30 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                 with self._assert_writer_session(makers) as session:
                     session.execute("test2")
 
+    def test_deprecated_async_reader_name(self):
+        if sys.version_info >= (3, 7):
+            self.skipTest("Test only runs on Python < 3.7")
+
+        context = oslo_context.RequestContext()
+
+        old = getattr(enginefacade.reader, "async")
+
+        @old
+        def go1(context):
+            context.session.execute("test1")
+
+        go1(context)
+
+        with self._assert_engines() as engines:
+            with self._assert_makers(engines) as makers:
+                with self._assert_async_reader_session(
+                        makers, assert_calls=False) as session:
+                    session.execute("test1")
+
     def test_async_reader_then_reader_ok(self):
         context = oslo_context.RequestContext()
 
-        @enginefacade.reader.async
+        @enginefacade.reader.async_
         def go1(context):
             context.session.execute("test1")
 
@@ -2161,7 +2209,7 @@ class ConfigOptionsTest(oslo_test_base.BaseTestCase):
 
         self.assertEqual(1, len(w))
         self.assertTrue(
-            issubclass(w[-1].category, exception.NotSupportedWarning))
+            issubclass(w[-1].category, warning.NotSupportedWarning))
         self.assertEqual(
             "Configuration option(s) ['fake1', 'wrong2'] not supported",
             str(w[-1].message)
